package de.juplo.plugins.hibernate;

/*
 * Copyright 2001-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.security.NoSuchAlgorithmException;
import java.util.EnumSet;
import java.util.Map;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.hibernate.boot.spi.MetadataImplementor;
import org.hibernate.cfg.AvailableSettings;
import org.hibernate.engine.config.spi.ConfigurationService;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.tool.schema.TargetType;
import org.hibernate.tool.schema.internal.ExceptionHandlerCollectingImpl;
import org.hibernate.tool.schema.internal.exec.ScriptTargetOutputToFile;
import org.hibernate.tool.schema.spi.ExecutionOptions;
import org.hibernate.tool.schema.spi.SchemaManagementTool;
import org.hibernate.tool.schema.spi.SchemaManagementToolCoordinator;
import org.hibernate.tool.schema.spi.ScriptTargetOutput;
import org.hibernate.tool.schema.spi.TargetDescriptor;


/**
 * Generate/Execute SQL to update the database-schema according to the
 * configured mappings.
 *
 * @goal update
 * @phase process-classes
 * @threadSafe
 * @requiresDependencyResolution runtime
 */
public class UpdateMojo extends AbstractSchemaMojo
{
  /**
   * Output file.
   * <p>
   * If the specified filename is not absolut, the file will be created
   * relative to the project build directory
   * (<code>project.build.directory</code>).
   *
   * @parameter property="hibernate.schema.update" default-value="update.sql"
   * @since 1.0
   */
  private String outputFile;


  @Override
  public final void execute()
    throws
      MojoFailureException,
      MojoExecutionException
  {
    try
    {
      super.execute(new MD5ModificationTracker(buildDirectory, outputFile, getLog()));
    }
    catch (NoSuchAlgorithmException e)
    {
      throw new MojoFailureException("Digest-Algorithm MD5 is missing!", e);
    }
  }


  @Override
  ExceptionHandlerCollectingImpl build(final MetadataImplementor metadata)
      throws
        MojoExecutionException,
        MojoFailureException
  {
    final ServiceRegistry registry =
        metadata.getMetadataBuildingOptions().getServiceRegistry();
    final Map settings = 
        registry.getService(ConfigurationService.class).getSettings();
    SchemaManagementTool tool = registry.getService(SchemaManagementTool.class);

    final EnumSet<TargetType> targetTypes = EnumSet.of(TargetType.SCRIPT);
    if (Boolean.parseBoolean(settings.get(EXECUTE).toString()))
      targetTypes.add(TargetType.DATABASE);

    TargetDescriptor target = new TargetDescriptor()
    {
      @Override
      public EnumSet<TargetType> getTargetTypes()
      {
        return targetTypes;
      }

      @Override
      public ScriptTargetOutput getScriptTargetOutput()
      {
        String charset =
            (String)settings.get(AvailableSettings.HBM2DDL_CHARSET_NAME);
        return new ScriptTargetOutputToFile(new File(outputFile), charset);
      }
    };

    ExceptionHandlerCollectingImpl handler =
        new ExceptionHandlerCollectingImpl();

    ExecutionOptions options =
        SchemaManagementToolCoordinator.buildExecutionOptions(
            registry
                .getService(ConfigurationService.class)
                .getSettings(),
            handler
            );

    Map config = options.getConfigurationValues();

    tool.getSchemaMigrator(config).doMigration(metadata, options, target);

    return handler;
  }
}
